#' function to simulate data for double observer data with misID when all animals are detectable
#' assumes 2 species and three types of observations (sp 1, sp 2, unknown)
#' @param N number of animals
#' @param pi1 proportion truly of species 1
#' @param p11 probability of correctly id'ing species 1
#' @param p12 probability of classifying sp 1 as species 2
#' @param p22 probability of classifying sp 2 correctly
#' @param p21 probability of classifying sp 2 as species 1
#' @return A vector giving # of animals with each type of sighting history (11,12,13,22,23,33)
sim_data_misID_simple<-function(N,pi1,p11,p12,p22,p21){
	n.sp1=round(N*pi1)
	n.sp2=N-n.sp1
	p13=1-p11-p12
	p23=1-p22-p21
	Probs1=c(p11*p11,2*p11*p12,2*p11*p13,p12*p12,2*p12*p13,p13*p13)
	Probs2=c(p21*p21,2*p21*p22,2*p21*p23,p22*p22,2*p22*p23,p23*p23)
	return(rmultinom(1,n.sp1,Probs1)+rmultinom(1,n.sp2,Probs2))	
}

#' function to simulate data for single observer data with misID when all animals are detectable
#' assumes 2 species and three types of observations (sp 1, sp 2, unknown)
#' @param N number of animals
#' @param pi1 proportion truly of species 1
#' @param p11 probability of correctly id'ing species 1
#' @param p12 probability of classifying sp 1 as species 2
#' @param p22 probability of classifying sp 2 correctly
#' @param p21 probability of classifying sp 2 as species 1
#' @return A vector giving # of animals with each type of sighting history 
sim_data_misID_1sp<-function(N,pi1,p11,p12,p22,p21){
  n.sp1=round(N*pi1)
  n.sp2=N-n.sp1
  p13=1-p11-p12
  p23=1-p22-p21
  Probs1=c(p11,p12,p13)
  Probs2=c(p21,p22,p23)
  return(rmultinom(1,n.sp1,Probs1)+rmultinom(1,n.sp2,Probs2))	
}

#' function to simulate data for double observer data with misID when all animals are detectable
#' assumes 2 species and two types of observations (sp 1, sp 2)
#' @param N number of animals
#' @param pi1 proportion truly of species 1
#' @param p11 probability of correctly id'ing species 1
#' @param p22 probability of classifying sp 2 correctly
#' @return A vector giving # of animals with each type of sighting history (11,12,22)
sim_data_misID_noU<-function(N,pi1,p11,p22){
  n.sp1=round(N*pi1)
  n.sp2=N-n.sp1
  Probs1=c(p11*p11,2*p11*(1-p11),(1-p11)*(1-p11))
  Probs2=c((1-p22)*(1-p22),2*p22*(1-p22),p22*p22)
  return(rmultinom(1,n.sp1,Probs1)+rmultinom(1,n.sp2,Probs2))
}

#' function to simulate data for double observer data with partial observation
#' assumes 2 species and three types of observations (sp 1, sp 2,unknown)
#' @param N number of animals
#' @param pi1 proportion truly of species 1
#' @param p11 probability of correctly id'ing species 1
#' @param p22 probability of classifying sp 2 correctly
#' @return A vector giving # of animals with each type of sighting history (11,13,22,23,33)
sim_data_misID_PO<-function(N,pi1,p11,p22){
  n.sp1=round(N*pi1)
  n.sp2=N-n.sp1
  Probs1=c(p11*p11,2*p11*(1-p11),0,0,(1-p11)*(1-p11))
  Probs2=c(0,0,p22*p22,2*p22*(1-p22),(1-p22)*(1-p22))
  return(rmultinom(1,n.sp1,Probs1)+rmultinom(1,n.sp2,Probs2))
}

#' function to compute 95% profile intervals for binomial distribution
#' @param N number of binomial trials
#' @param n number of binomial successes
#' @return Vector of lower and upper CI limits
#' @export
#' @keywords binomial, profile likelihood
#' @author Paul B. Conn
prof_lik_binom<-function(N,n){
  mle=n/N
  max.logL=dbinom(n,N,mle,log=TRUE)
  bin_logL<-function(p,N,n,max.logL){
    abs(max.logL-1.92-dbinom(n,N,p,log=TRUE))
  }
  #lower
  lower=optimize(f=bin_logL,interval=c(0,mle),n=n,N=N,max.logL=max.logL)
  upper=optimize(f=bin_logL,interval=c(mle,1),n=n,N=N,max.logL=max.logL)
  return(c(lower$minimum,upper$minimum))
}


#' @param Dat dataset for double observer data with misID when all animals are detectable
#' @param symm If TRUE, assumes that psi_{12}=psi_{21}
#' @return an MCMC chain for pi
#' @export
#' @keywords MCMC, misidentification
#' @author Paul B. Conn
MCMC_simple<-function(Dat,symm=FALSE){
	require(MCMCpack)
	N.tot=sum(Dat)
	n.indiv=N.tot
	prior.phi=c(1,1)
	n.iter=12000
	prior.phi=c(1,1)
	prior.mean.mu=matrix(0,4,3)
	#prior.mean.mu[c(1,3),1]=c(1.8,1.8)
	#prior.mean.mu[c(2,4),1]=c(1.1,1.1)
	prior.mean.mu[c(1,3),1]=0
	prior.mean.mu[c(2,4),1]=0
	prior.mean.sd=matrix(100,4,3)
	prior.mean.sd[c(1,3),1]=1.75
	prior.mean.sd[c(2,4),1]=1.75
	mh.sd=0.2
	MCMC=list(misID=array(0,dim=c(n.iter,4,1)),phi=rep(0,n.iter))
	
	
	#Latent=c(rep(1,round(n*phi*delta11^2)),rep(2,round(n*(1-phi)*delta21^2)),rep(1,round(N[2]*phi)),rep(2,round(N[2]*(1-phi))),rep(2,round((1-phi)*delta22^2*n)),rep(1,round(phi*delta12^2*n)))
	Data=matrix(0,n.indiv,2)
	if(Dat[1]>0)Data[1:Dat[1],]=1
	if(Dat[2]>0)Data[(Dat[1]+1):sum(Dat[1:2]),1]=1
	if(Dat[2]>0)Data[(Dat[1]+1):sum(Dat[1:2]),2]=2
	if(Dat[3]>0)Data[(sum(Dat[1:2])+1):sum(Dat[1:3]),1]=1
	if(Dat[3]>0)Data[(sum(Dat[1:2])+1):sum(Dat[1:3]),2]=3	
	if(Dat[4]>0)Data[(sum(Dat[1:3])+1):sum(Dat[1:4]),]=2
	if(Dat[5]>0)Data[(sum(Dat[1:4])+1):sum(Dat[1:5]),1]=2
	if(Dat[5]>0)Data[(sum(Dat[1:4])+1):sum(Dat[1:5]),2]=3
	if(Dat[6]>0)Data[(sum(Dat[1:5])+1):sum(Dat[1:6]),]=3
	
	#c(rep(1,Dat[1]),rep(2,Dat[2]),rep(3,Dat[3]),rep(4,Dat[4]),rep(5,Dat[5]),rep(6,Dat[6]))

	misID.mat=matrix(0,2,3)
	misID.mat[1,1]=1
	misID.mat[1,2]=-1
	misID.mat[1,3]=2
	misID.mat[2,1]=-1
	misID.mat[2,2]=3
	misID.mat[2,3]=4
	if(symm==TRUE)misID.mat[2,1]=misID.mat[1,2]
	misID.formulas=c(~1,~1,~1,~1)
	
	
	######## estimation routine
	Par=list(misID=matrix(0,4,1),phi=0.4)
	Par$misID[c(1,3),1]=c(2,3)
	Par$phi=c(.6,.4)
	#Par$phi=c(runif(1)*0.5+.5,0) #initial phi is U(0.5,1)
	#Par$phi[2]=1-Par$phi[1]
	MCMC$misID[1,,]=Par$misID
	MCMC$phi[1]=Par$phi[1]
	Probs=matrix(0,n.indiv,2)
	Latent=sample(c(1,2),N.tot,replace=TRUE) #holds Tk
	
	subset_array<-function(Array,row,Col,n.indiv){  #2nd dimension fixed, 3rd dimension set to a particular value for each index of 1st dimension (result is vector)
		Cur=rep(0,n.indiv)
		for(iind in 1:n.indiv)Cur[iind]=Array[iind,row,Col[iind]]
		Cur
	}
	library(compiler)
	subset_array<-cmpfun(subset_array)

	subset_conf_array<-function(Confuse,Latent,Cur.dat,n.indiv){
		Cur=rep(0,n.indiv)
		for(iind in 1:n.indiv)Cur[iind]=Confuse[iind,Latent[iind],Cur.dat[iind]]
		Cur
	}
	subset_conf_array<-cmpfun(subset_conf_array)

	get_confusion_array<-function(Confuse,Cov=NULL,Beta,n.indiv,misID.mat,misID.formulas,symm=TRUE){
		if(is.null(Cov)==1)Cov=data.frame(matrix(1,n.indiv,1))
		DM=vector("list",max(misID.mat))
		Pi=DM
		ind.mat=matrix(c(1:length(misID.mat)),nrow(misID.mat),ncol(misID.mat))
		
		for(ipar in 1:length(misID.mat)){
			if(misID.mat[ipar]==0)Pi[[ipar]]=rep(0,n.indiv)
			if(misID.mat[ipar]<0)Pi[[ipar]]=rep(1,n.indiv)
			if(misID.mat[ipar]>0){
				DM[[ipar]]=model.matrix(misID.formulas[[misID.mat[ipar]]],data=Cov)
				Pi[[ipar]]=exp(DM[[ipar]]%*%Beta[misID.mat[ipar],])
			}
		}
		if(symm==TRUE){
			for(iind in 1:n.indiv){
					for(icol in 1:ncol(misID.mat)){
						Confuse[iind,1,icol]=Pi[[ind.mat[1,icol]]][iind]
					}
					Confuse[iind,1,]=Confuse[iind,1,]/sum(Confuse[iind,1,])
					Pi[[ind.mat[2,3]]]=rep(1,n.indiv)
					Pi[[ind.mat[2,1]]]=(Confuse[iind,1,2]+Confuse[iind,1,2]*Pi[[ind.mat[2,2]]])/(1-Confuse[iind,1,2])
					for(icol in 1:ncol(misID.mat))Confuse[iind,2,icol]=Pi[[ind.mat[2,icol]]][iind]
					Confuse[iind,2,]=Confuse[iind,2,]/sum(Confuse[iind,2,])		
			}
		}
		else{
			for(iind in 1:n.indiv){
				for(irow in 1:nrow(misID.mat)){
					for(icol in 1:ncol(misID.mat))Confuse[iind,irow,icol]=Pi[[ind.mat[irow,icol]]][iind]
					Confuse[iind,irow,]=Confuse[iind,irow,]/sum(Confuse[iind,irow,])
				}
			}
		}
		Confuse
	}
	get_confusion_array<-cmpfun(get_confusion_array)
	Confuse=array(0,dim=c(n.indiv,dim(misID.mat)))
	Confuse.temp=Confuse
	Confuse=get_confusion_array(Confuse,Cov=NULL,Beta=Par$misID,n.indiv=n.indiv,misID.mat=misID.mat,misID.formulas=misID.formulas,symm=symm)

	for(iiter in 2:n.iter){
		### update latent states 
		Probs[,1]=Par$phi[1]*subset_array(Confuse,1,Data[,1],n.indiv)*subset_array(Confuse,1,Data[,2],n.indiv)
		Probs[,2]=Par$phi[2]*subset_array(Confuse,2,Data[,1],n.indiv)*subset_array(Confuse,2,Data[,2],n.indiv)
		Probs=Probs/rowSums(Probs)
		for(iind in 1:n.indiv)Latent[iind]=sample(c(1,2),1,prob=Probs[iind,],replace=TRUE)
		### update phi
		Par$phi=rdirichlet(1,c(prior.phi[1]+sum(Latent==1),prior.phi[2]+sum(Latent==2)))
		MCMC$phi[iiter]=Par$phi[1]
		### update misID parameters
		old.logL=sum(log(subset_conf_array(Confuse,Latent,Data[,1],n.indiv)))+sum(log(subset_conf_array(Confuse,Latent,Data[,2],n.indiv)))
		Temp=Par$misID
		for(irow in 1:4){
			for(icol in 1:1){
				Temp[irow,icol]=Par$misID[irow,icol]+runif(1,-mh.sd,mh.sd)
				Confuse.temp=get_confusion_array(Confuse=Confuse,Cov=NULL,Beta=Temp,n.indiv=n.indiv,misID.mat=misID.mat,misID.formulas=misID.formulas,symm=symm)
				new.logL=sum(log(subset_conf_array(Confuse.temp,Latent,Data[,1],n.indiv)))+sum(log(subset_conf_array(Confuse.temp,Latent,Data[,2],n.indiv)))
				if(mean(Confuse.temp[,1,1])>mean(Confuse.temp[,1,2]) & mean(Confuse.temp[,1,1])>mean(Confuse.temp[,1,3]) & mean(Confuse.temp[,2,2])>mean(Confuse.temp[,2,1]) & mean(Confuse.temp[,2,2])>mean(Confuse.temp[,2,3])){
					if(runif(1)<exp(new.logL+dnorm(Temp[irow,icol],prior.mean.mu[irow,icol],prior.mean.sd[irow,icol],log=1)-old.logL-dnorm(Par$misID[irow,icol],prior.mean.mu[irow,icol],prior.mean.sd[irow,icol],log=1))){
						old.logL=new.logL
						Par$misID[irow,icol]=Temp[irow,icol]
						Confuse=Confuse.temp
					}
					else{
						Temp=Par$misID
					}
				}
				else Temp=Par$misID
			}
		}
		MCMC$misID[iiter,,]=Par$misID
	}
	return(MCMC)
}


#' @param Dat dataset for single observer data with misID when all animals are detectable
#' @param Gold.data dataset for gold standard data; true species on row, observed species on column
#' @return an MCMC chain for pi
#' @export
#' @keywords MCMC, misidentification
#' @author Paul B. Conn
MCMC_gold_standard<-function(Dat,Gold.data){
  require(MCMCpack)
  n.indiv=sum(Dat)
  prior.phi=c(1,1)
  n.iter=12000
  prior.phi=c(1,1)
  prior.mean.mu=matrix(0,4,3)
  #prior.mean.mu[c(1,3),1]=c(1.8,1.8)
  #prior.mean.mu[c(2,4),1]=c(1.1,1.1)
  prior.mean.mu[c(1,3),1]=0
  prior.mean.mu[c(2,4),1]=0
  prior.mean.sd=matrix(100,4,3)
  prior.mean.sd[c(1,3),1]=1.75
  prior.mean.sd[c(2,4),1]=1.75
  mh.sd=0.2
  MCMC=list(misID=array(0,dim=c(n.iter,4,1)),phi=rep(0,n.iter))
  
  Latent=matrix(0,2,3)  #set intial values of latent state distribution
  phi.guess=Dat[1]/(Dat[1]+Dat[2])
  Latent[1,]=rbinom(3,Dat,phi.guess)
  Latent[2,]=Dat-Latent[1,]
   
  get_confusion_mat<-function(Beta){
    Cur.mat=matrix(0,2,3)
    Cur.mat[,1:2]=Beta
    Cur.mat=exp(Cur.mat)
    Cur.mat[1,]=Cur.mat[1,]/sum(Cur.mat[1,])
    Cur.mat[2,]=Cur.mat[2,]/sum(Cur.mat[2,])
    Cur.mat
  }
    
  ######## estimation routine
  Par=list(misID=matrix(0,4,1),phi=0.4)
  Par$misID[c(1,4),1]=c(2,3)
  #Par$phi=c(.6,.4)
  Par$phi=c(runif(1)*0.5+.5,0) #initial phi is U(0.5,1)
  Par$phi[2]=1-Par$phi[1]
  MCMC$misID[1,,]=Par$misID
  MCMC$phi[1]=Par$phi[1]
    
  Confuse=get_confusion_mat(Par$misID)
   
  for(iiter in 2:n.iter){
    ### update latent states 
    Probs=Confuse*Par$phi
    for(icol in 1:3){
      n.succ=rbinom(1,Dat[icol],prob=Probs[1,icol]/sum(Probs[,icol]))
      Latent[,icol]=c(n.succ,Dat[icol]-n.succ)
    }
    ### update phi
    Par$phi=as.numeric(rdirichlet(1,c(prior.phi[1]+sum(Latent[1,]),prior.phi[2]+sum(Latent[2,]))))
    MCMC$phi[iiter]=Par$phi[1]
    ### update misID parameters
    old.logL=dmultinom(Latent[1,],prob=Confuse[1,],log=TRUE)+dmultinom(Latent[2,],prob=Confuse[2,],log=TRUE)+
      dmultinom(Gold.data[1,],prob=Confuse[1,],log=TRUE)+dmultinom(Gold.data[2,],prob=Confuse[2,],log=TRUE)
    Temp=Par$misID
    for(ipar in 1:4){
      Temp[ipar]=Par$misID[ipar]+runif(1,-mh.sd,mh.sd)
      Confuse.temp=get_confusion_mat(Temp)
      new.logL=dmultinom(Latent[1,],prob=Confuse.temp[1,],log=TRUE)+dmultinom(Latent[2,],prob=Confuse.temp[2,],log=TRUE)+
        dmultinom(Gold.data[1,],prob=Confuse.temp[1,],log=TRUE)+dmultinom(Gold.data[2,],prob=Confuse.temp[2,],log=TRUE)
      if(Confuse.temp[1,1]>Confuse.temp[1,2] & Confuse.temp[1,1]>Confuse.temp[1,3] & Confuse.temp[2,2]>Confuse.temp[2,1] & Confuse.temp[2,2]>Confuse.temp[2,3]){
        if(runif(1)<exp(new.logL+dnorm(Temp[ipar],prior.mean.mu[ipar],prior.mean.sd[ipar],log=1)-old.logL-dnorm(Par$misID[ipar],prior.mean.mu[ipar],prior.mean.sd[ipar],log=1))){
          old.logL=new.logL
          Par$misID[ipar]=Temp[ipar]
          Confuse=Confuse.temp
        }
        else{
          Temp=Par$misID
        }
      }
      else Temp=Par$misID
    }
    MCMC$misID[iiter,,]=Par$misID
  }
  return(MCMC)
}


#' @param Dat dataset for single observer data with misID when all animals are detectable
#' @return an MCMC chain for pi
#' @export
#' @keywords MCMC, misidentification
#' @author Paul B. Conn
MCMC_noU<-function(Dat){
  require(MCMCpack)
  n.indiv=sum(Dat)
  prior.phi=c(1,1)
  n.iter=12000
  prior.phi=c(1,1)
  prior.mean.mu=0
  prior.mean.sd=1.75
  mh.sd=0.2
  MCMC=list(misID=array(0,dim=c(n.iter,2,1)),phi=rep(0,n.iter))
  
  Latent=matrix(0,2,3)  #set intial values of latent state distribution
  phi.guess=Dat[1]/(Dat[1]+Dat[3])
  Latent[1,]=round(Dat*phi.guess)
  Latent[2,]=Dat-Latent[1,]
  
  get_confusion_mat<-function(Beta){
    Cur.mat=matrix(0,2,2)
    Cur.mat[1,1]=Beta[1]
    Cur.mat[2,2]=Beta[2]
    Cur.mat=exp(Cur.mat)
    Cur.mat[1,]=Cur.mat[1,]/sum(Cur.mat[1,])
    Cur.mat[2,]=Cur.mat[2,]/sum(Cur.mat[2,])
    Cur.mat
  }
  
  ######## estimation routine
  Par=list(misID=matrix(0,2,1),phi=0.4)
  Par$misID[c(1,2),1]=c(2,3)
  #Par$phi=c(.6,.4)
  Par$phi=c(runif(1)*0.5+.5,0) #initial phi is U(0.5,1)
  Par$phi[2]=1-Par$phi[1]
  MCMC$misID[1,,]=Par$misID
  MCMC$phi[1]=Par$phi[1]
  
  Confuse=get_confusion_mat(Par$misID)
  Obs.prob=matrix(0,2,3)
  Obs.prob[1,]=c(Confuse[1,1]^2,2*Confuse[1,1]*Confuse[1,2],Confuse[1,2]^2)
  Obs.prob[2,]=c(Confuse[2,1]^2,2*Confuse[2,1]*Confuse[2,2],Confuse[2,2]^2)
  Obs.prob.temp=Obs.prob
  
  for(iiter in 2:n.iter){
    ### update latent states 
    Probs=Obs.prob*Par$phi
    for(icol in 1:3){
      n.succ=rbinom(1,Dat[icol],prob=Probs[1,icol]/sum(Probs[,icol]))
      Latent[,icol]=c(n.succ,Dat[icol]-n.succ)
    }
    ### update phi
    Par$phi=as.numeric(rdirichlet(1,c(prior.phi[1]+sum(Latent[1,]),prior.phi[2]+sum(Latent[2,]))))
    MCMC$phi[iiter]=Par$phi[1]
    ### update misID parameters
    old.logL=dmultinom(Latent[1,],prob=Obs.prob[1,],log=TRUE)+dmultinom(Latent[2,],prob=Obs.prob[2,],log=TRUE)
    Temp=Par$misID
    for(ipar in 1:2){
      Temp[ipar]=Par$misID[ipar]+runif(1,-mh.sd,mh.sd)
      Confuse.temp=get_confusion_mat(Temp)
      Obs.prob.temp[1,]=c(Confuse.temp[1,1]^2,2*Confuse.temp[1,1]*Confuse.temp[1,2],Confuse.temp[1,2]^2)
      Obs.prob.temp[2,]=c(Confuse.temp[2,1]^2,2*Confuse.temp[2,1]*Confuse.temp[2,2],Confuse.temp[2,2]^2)
      new.logL=dmultinom(Latent[1,],prob=Obs.prob.temp[1,],log=TRUE)+dmultinom(Latent[2,],prob=Obs.prob.temp[2,],log=TRUE)
      if(Confuse.temp[1,1]>Confuse.temp[1,2] & Confuse.temp[2,2]>Confuse.temp[2,1]){
        if(runif(1)<exp(new.logL+dnorm(Temp[ipar],prior.mean.mu,prior.mean.sd,log=1)-old.logL-dnorm(Par$misID[ipar],prior.mean.mu,prior.mean.sd,log=1))){
          old.logL=new.logL
          Par$misID[ipar]=Temp[ipar]
          Confuse=Confuse.temp
          Obs.prob=Obs.prob.temp
        }
        else{
          Temp=Par$misID
        }
      }
      else Temp=Par$misID
    }
    MCMC$misID[iiter,,]=Par$misID
  }
  return(MCMC)
}

#' @param Dat dataset for single observer data with misID when all animals are detectable
#' @return an MCMC chain for pi
#' @export
#' @keywords MCMC, misidentification
#' @author Paul B. Conn
MCMC_PO<-function(Dat){
  require(MCMCpack)
  n.indiv=sum(Dat)
  prior.phi=c(1,1)
  n.iter=12000
  prior.phi=c(1,1)
  prior.mean.mu=0
  prior.mean.sd=1.75
  mh.sd=0.2
  MCMC=list(misID=array(0,dim=c(n.iter,2,1)),phi=rep(0,n.iter))
  
  Latent=matrix(0,2,5)  #set intial values of latent state distribution
  phi.guess=Dat[1]/(Dat[1]+Dat[3])
  Latent[1,1:2]=Dat[1:2]
  Latent[2,3:4]=Dat[3:4]
  Latent[1,5]=round(Dat[5]*phi.guess)
  Latent[2,5]=Dat[5]-Latent[1,5]
  
  get_confusion_mat<-function(Beta){
    Cur.mat=matrix(0,2,3)
    Cur.mat[1,1]=Beta[1]
    Cur.mat[2,2]=Beta[2]
    Cur.mat[1,2]=-1000
    Cur.mat[2,1]=-1000
    Cur.mat=exp(Cur.mat)
    Cur.mat[1,]=Cur.mat[1,]/sum(Cur.mat[1,])
    Cur.mat[2,]=Cur.mat[2,]/sum(Cur.mat[2,])
    Cur.mat
  }
  
  ######## estimation routine
  Par=list(misID=matrix(0,2,1),phi=0.4)
  Par$misID[c(1,2),1]=c(2,3)
  #Par$phi=c(.6,.4)
  Par$phi=c(runif(1)*0.5+.5,0) #initial phi is U(0.5,1)
  Par$phi[2]=1-Par$phi[1]
  MCMC$misID[1,,]=Par$misID
  MCMC$phi[1]=Par$phi[1]
  
  Confuse=get_confusion_mat(Par$misID)
  Obs.prob=matrix(0,2,5)
  Obs.prob[1,]=c(Confuse[1,1]^2,2*Confuse[1,1]*Confuse[1,3],0,0,Confuse[1,3]^2)
  Obs.prob[2,]=c(0,0,Confuse[2,2]^2,2*Confuse[2,2]*Confuse[2,3],Confuse[2,3]^2)
  Obs.prob.temp=Obs.prob
  
  for(iiter in 2:n.iter){
    ### update latent states 
    Probs=Obs.prob*Par$phi
    for(icol in 5:5){
      n.succ=rbinom(1,Dat[icol],prob=Probs[1,icol]/sum(Probs[,icol]))
      Latent[,icol]=c(n.succ,Dat[icol]-n.succ)
    }
    ### update phi
    Par$phi=as.numeric(rdirichlet(1,c(prior.phi[1]+sum(Latent[1,]),prior.phi[2]+sum(Latent[2,]))))
    MCMC$phi[iiter]=Par$phi[1]
    ### update misID parameters
    old.logL=dmultinom(Latent[1,],prob=Obs.prob[1,],log=TRUE)+dmultinom(Latent[2,],prob=Obs.prob[2,],log=TRUE)
    Temp=Par$misID
    for(ipar in 1:2){
      Temp[ipar]=Par$misID[ipar]+runif(1,-mh.sd,mh.sd)
      Confuse.temp=get_confusion_mat(Temp)
      Obs.prob.temp[1,]=c(Confuse.temp[1,1]^2,2*Confuse.temp[1,1]*Confuse.temp[1,3],0,0,Confuse.temp[1,3]^2)
      Obs.prob.temp[2,]=c(0,0,Confuse.temp[2,2]^2,2*Confuse.temp[2,2]*Confuse.temp[2,3],Confuse.temp[2,3]^2)
      new.logL=dmultinom(Latent[1,],prob=Obs.prob.temp[1,],log=TRUE)+dmultinom(Latent[2,],prob=Obs.prob.temp[2,],log=TRUE)
      if(Confuse.temp[1,1]>Confuse.temp[1,2] & Confuse.temp[2,2]>Confuse.temp[2,1]){
        if(runif(1)<exp(new.logL+dnorm(Temp[ipar],prior.mean.mu,prior.mean.sd,log=1)-old.logL-dnorm(Par$misID[ipar],prior.mean.mu,prior.mean.sd,log=1))){
          old.logL=new.logL
          Par$misID[ipar]=Temp[ipar]
          Confuse=Confuse.temp
          Obs.prob=Obs.prob.temp
        }
        else{
          Temp=Par$misID
        }
      }
      else Temp=Par$misID
    }
    MCMC$misID[iiter,,]=Par$misID
  }
  return(MCMC)
}
